@@ -61,6 +61,7 @@ namespace eval ::logcanvas {
       set sel_rev(B) {}
       variable logcanvas ".logcanvas$my_idx"
 
+      gen_log:log T "ENTER [namespace current]"
       set sys_loc [split $how {,}]
       set sys [lindex $sys_loc 0]
       set loc [lindex $sys_loc 1]
@@ -73,7 +74,9 @@ namespace eval ::logcanvas {
         catch {$logcanvas.canvas itemconfigure Sel$AorB -fill gray90}
         $logcanvas.canvas dtag Sel$AorB
         $logcanvas.up.rev${AorB}_rvers configure -text {}
+        $logcanvas.up.log${AorB}_rlogfm.rcomment configure -state normal
         $logcanvas.up.log${AorB}_rlogfm.rcomment delete 1.0 end
+        $logcanvas.up.log${AorB}_rlogfm.rcomment configure -state disabled
         $logcanvas.up.rev${AorB}_rwho configure -text {}
         $logcanvas.up.rev${AorB}_rdate configure -text {}
         set sel_tag($AorB) {}
@@ -106,7 +109,9 @@ namespace eval ::logcanvas {
           $logcanvas.up.rev${AorB}_rwho configure -text $revwho($rev)
           $logcanvas.up.rev${AorB}_rdate configure -text\
               "$revdate($rev) $revtime($rev)"
+          $logcanvas.up.log${AorB}_rlogfm.rcomment configure -state normal
           $logcanvas.up.log${AorB}_rlogfm.rcomment insert end $revcomment($rev)
+          $logcanvas.up.log${AorB}_rlogfm.rcomment configure -state disabled
         }
         $logcanvas.canvas addtag Sel$AorB withtag rect$rev
         $logcanvas.canvas itemconfigure SelA -fill $cvscfg(colourA)
@@ -140,6 +145,7 @@ namespace eval ::logcanvas {
 
         switch -- $sys {
           "SVN" {
+            # Find out if it's a directory, if we can
             set kind ""
             set info_cmd [exec::new "svn info \"[file tail $fname]\""]
             set info_lines [split [$info_cmd\::output] "\n"]
@@ -155,6 +161,12 @@ namespace eval ::logcanvas {
             $logcanvas.up.rfname delete 0 end
             $logcanvas.up.rfname insert end "$fname"
             $logcanvas.up.rfname configure -state readonly -bg $cvsglb(textbg)
+            $logcanvas.log configure \
+                -command [namespace code {
+                    # FIXME: we get the rev and dont use it
+                    set rev [$logcanvas.up.revA_rvers cget -text] 
+                    svn_log $filename
+                 }]
             if {$kind == "directory"} {
               $logcanvas.diff configure -state disabled
               $logcanvas.annotate configure -state disabled
@@ -176,16 +188,20 @@ namespace eval ::logcanvas {
                    set A [string trimleft $revA {r}]
                    set B [string trimleft $revB {r}]
                    if {$revB == ""} {
-                     comparediff_files "$revpath($revA)@$A" [file tail $revpath($revA)]
+                     comparediff_files $logcanvas "$revpath($revA)@$A" [file tail $revpath($revA)]
                    } else {
-                     comparediff_files "$revpath($revA)@$A" "$revpath($revB)@$B"
+                     comparediff_files $logcanvas "$revpath($revA)@$A" "$revpath($revB)@$B"
                    }
                 }]
               $logcanvas.annotate configure \
                  -command [namespace code {
                    set rev [$logcanvas.up.revA_rvers cget -text]
                    set R [string trimleft $rev {r}]
-                   svn_annotate_r $R $revpath($rev)
+                   if {$R == ""} {
+                     svn_annotate_r $R $filename
+                   } else {
+                     svn_annotate_r $R $revpath($rev)
+                   }
                  }]
             }
             $logcanvas.delta configure \
@@ -219,13 +235,16 @@ namespace eval ::logcanvas {
            $logcanvas.up.rfname delete 0 end
            $logcanvas.up.rfname insert end "$fname,v"
            $logcanvas.up.rfname configure -state readonly -bg $cvsglb(textbg)
-
            if {$loc == "rep"} {
              # Working on repository files, not checked out
              $logcanvas.view configure \
                 -command [namespace code {
                   cvs_fileview_checkout [$logcanvas.up.revA_rvers cget -text] $filename
                 }]
+             $logcanvas.log configure \
+                  -command [namespace code {
+                    cvs_filelog $filename $logcanvas 0
+                  }]
              $logcanvas.annotate configure \
                 -command [namespace code {
                    cvs_annotate_r [$logcanvas.up.revA_rvers cget\
@@ -240,6 +259,10 @@ namespace eval ::logcanvas {
              $logcanvas.delta configure -state disabled
            } else {
              # We have a checked-out local file
+             $logcanvas.log configure \
+                  -command [namespace code {
+                    cvs_log $filename
+                  }]
              $logcanvas.view configure \
                -command [namespace code {
                   cvs_fileview_update [$logcanvas.up.revA_rvers cget -text] \
@@ -279,6 +302,8 @@ namespace eval ::logcanvas {
            $logcanvas.up.rfname configure -state readonly -bg $cvsglb(textbg)
            $logcanvas.view configure -state disabled
            $logcanvas.annotate configure -state disabled
+           $logcanvas.log configure -command [namespace code \
+                                               {rcs_log $filename}]
            $logcanvas.delta configure -state disabled
           }
         }
@@ -1097,7 +1122,7 @@ namespace eval ::logcanvas {
             append rev_info {$revdate($revision) }
           }
           if {$opt(show_box_revwho)} {
-            append rev_info {$revwho($revision) }
+            append rev_info {"$revwho($revision)" }
           }
           if {$opt(show_box_rev)} {
             append rev_info {$revision}
@@ -1281,7 +1306,7 @@ namespace eval ::logcanvas {
          -menu $logcanvas.menubar.file -underline 0
       menu $logcanvas.menubar.file -tearoff 0
       $logcanvas.menubar.file add command -label "Shell window" -underline 0 \
-        -command {eval exec $cvscfg(shell) >& $cvscfg(null) &}
+        -command {exec::new $cvscfg(shell)}
       $logcanvas.menubar.file add separator
       $logcanvas.menubar.file add command -label "Close" -underline 0 \
         -command [namespace code {$logcanvas.close invoke}]
@@ -1442,7 +1467,7 @@ namespace eval ::logcanvas {
          
         frame $logcanvas.up.log${fm}_rlogfm -bd 3 -bg $cvscfg(colour$fm)
         text  $logcanvas.up.log${fm}_rlogfm.rcomment -height 5 \
-           -fg $cvsglb(textfg) -bg $cvsglb(textbg)\
+           -fg $cvsglb(textfg) -bg $cvsglb(textbg) -state disabled \
            -yscrollcommand [namespace code\
            "$logcanvas.up.log${fm}_rlogfm.yscroll set"]
            scrollbar $logcanvas.up.log${fm}_rlogfm.yscroll \
@@ -1499,6 +1524,7 @@ namespace eval ::logcanvas {
                  $scope\::reloadLog
                }]
       button $logcanvas.view -image Fileview
+      button $logcanvas.log -image Log
       button $logcanvas.annotate -image Annotate
       button $logcanvas.diff -image Diff \
         -command [namespace code {
@@ -1531,11 +1557,17 @@ namespace eval ::logcanvas {
                  namespace delete [namespace current]
                  exit_cleanup 0
                }]
+      button $logcanvas.stop -text "Stop" -bg red4 -fg white \
+        -activebackground red4 -activeforeground white \
+        -state [expr {$cvscfg(allow_abort) ? {normal} : {disabled}}] \
+        -command "$scope\::abortLog"
+
       pack $logcanvas.refresh \
         -in $logcanvas.down -side left \
         -ipadx 4 -ipady 4
       pack $logcanvas.down.btnfm -side left -fill y -expand 1
       pack $logcanvas.view \
+           $logcanvas.log \
            $logcanvas.annotate \
            $logcanvas.diff \
            $logcanvas.delta \
@@ -1555,6 +1587,8 @@ namespace eval ::logcanvas {
         {"Open the Repository Browser"}
       set_tooltips $logcanvas.view \
         {"View a version of the file"}
+      set_tooltips $logcanvas.log \
+        {"Revision Log of the file"}
       set_tooltips $logcanvas.annotate \
         {"View revision where each line was modified"}
       set_tooltips $logcanvas.diff \
