@@ -97,7 +97,7 @@ proc cvs_workdir_status {} {
 
   set cmd(cvs_status) [exec::new "$cvs -n -q status -l"]
   set status_lines [split [$cmd(cvs_status)\::output] "\n"]
-  if {$cvscfg(econtrol)} {
+  if {$cvscfg(showeditcol)} {
     set cmd(cvs_editors) [exec::new "$cvs -n -q editors -l"]
     set editors_lines [split [$cmd(cvs_editors)\::output] "\n"]
   }
@@ -110,6 +110,7 @@ proc cvs_workdir_status {} {
     # Status: or Sticky Tag:, putting each file's info (name, status, and tag)
     # into an array.
 
+    $cmd(cvs_status)\::destroy
     catch {unset cmd(cvs_status)}
     foreach logline $status_lines {
       if {[string match "File:*" $logline]} {
@@ -140,9 +141,17 @@ proc cvs_workdir_status {} {
             [string match "Result *" $date]} {
           ; # Leave as is
         } else {
-          set juliandate [clock scan $date -gmt yes]
-          set date [clock format $juliandate -format $cvscfg(dateformat)]
-          set Filelist($filename:date) $date
+          # CVS outputs time strings tcl can't handle, such as
+          # ones with +0100.  Let's discard them rather than
+          # trying to convert them.
+          set ret [catch {clock scan $date -gmt yes} ans]
+          if {$ret == 0} {
+            set juliandate $ans
+            set date [clock format $juliandate -format $cvscfg(dateformat)]
+            set Filelist($filename:date) $date
+          } else {
+            gen_log:log E "$ans"
+          }
         }
         set Filelist($filename:wrev) $revision
         set Filelist($filename:status) $status
@@ -175,6 +184,7 @@ proc cvs_workdir_status {} {
 
   if {[info exists cmd(cvs_editors)]} {
     set filename {}
+    $cmd(cvs_editors)\::destroy
     catch {unset cmd(cvs_editors)}
     foreach logline $editors_lines {
       set line [split $logline "\t"]
@@ -205,6 +215,7 @@ proc cvs_workdir_status {} {
   if {[info exists cmd(cvs_lockers)]} {
     set filename {}
     set lockers {}
+    $cmd(cvs_lockers)\::destroy
     catch {unset cmd(cvs_lockers)}
     foreach line $lockers_lines {
       if {[string match "Working file: *" $line]} {
@@ -258,10 +269,7 @@ proc cvs_remove {args} {
   }
 
   set cmd [exec::new "$cvs remove $filelist"]
-  if {$cvscfg(auto_status)} {
-    $cmd\::wait
-    setup_dir
-  }
+  auto_setup_dir $cmd
 
   gen_log:log T "LEAVE"
 }
@@ -303,6 +311,7 @@ proc cvs_remove_dir {args} {
       set commandline "$cvs remove \"$file\""
       $v\::do "$commandline" 1 status_colortags
       $v\::wait
+      $v\::clean_exec
     }
   }
 
@@ -434,10 +443,7 @@ proc cvs_add {binflag args} {
     append filelist [glob -nocomplain $cvscfg(aster) .??*]
   }
   set cmd [exec::new "$cvs add $binflag $filelist"]
-  if {$cvscfg(auto_status)} {
-    $cmd\::wait
-    setup_dir
-  }
+  auto_setup_dir $cmd
 
   gen_log:log T "LEAVE"
 }
@@ -472,6 +478,7 @@ proc cvs_add_dir {binflag args} {
       set commandline "$cvs add \"$file\""
       $v\::do "$commandline"
       $v\::wait
+      $v\::clean_exec
 
       cd $file
       gen_log:log F "CD [pwd]"
@@ -503,6 +510,8 @@ proc add_subdirs {binflag v} {
       set commandline "$cvs add \"$child\""
       $v\::do "$commandline"
       $v\::wait
+      $v\::clean_exec
+
       set awd [pwd]
       cd $child
       gen_log:log F "CD [pwd]"
@@ -579,9 +588,6 @@ proc rem_subdirs { v } {
       file delete -force -- $file
       if {[file exists $file]} {cvsfail "Remove $file failed" .workdir}
     }
-    #set commandline "$cvs remove $plainfiles"
-    #$v\::do "$commandline" 1
-    #$v\::wait
   }
 
   gen_log:log T "LEAVE"
@@ -638,6 +644,8 @@ proc cvs_log {args} {
   global cvs
   global cvscfg
 
+  gen_log:log T "ENTER ($args)"
+
   set filelist [join $args]
 
   # Don't recurse
@@ -655,7 +663,6 @@ proc cvs_log {args} {
 
   set logcmd [viewer::new "CVS log ($cvscfg(ldetail))"]
   $logcmd\::do "$commandline" 0 hilight_rcslog
-  busy_done .workdir.main
 
   gen_log:log T "LEAVE"
 }
@@ -941,11 +948,7 @@ proc cvs_update {tagname normal_binary action_if_no_tag get_all_dirs dir args} {
 
     set co_cmd [viewer::new "CVS Update"]
     $co_cmd\::do $commandline 0 status_colortags
-    
-    if {$cvscfg(auto_status)} {
-      $co_cmd\::wait
-      setup_dir
-    }
+    auto_setup_dir $co_cmd
   }
   gen_log:log T "LEAVE"
 }
@@ -1042,6 +1045,7 @@ proc cvs_status {args} {
   # support verious levels of verboseness. Ideas derived from GIC
   set statcmd [exec::new "$cvs -Q status $cmd_options $filelist"]
   set raw_status [$statcmd\::output]
+  $statcmd\::destroy
 
   if {$cvscfg(rdetail) == "verbose"} {
     view_output::new "CVS Status ($cvscfg(rdetail))" $raw_status
@@ -1153,7 +1157,7 @@ proc cvs_checkout { dir cvsroot prune kflag revtag date target mtag1 mtag2 modul
   return
 }
 
-proc cvs_filelog {filename parent} {
+proc cvs_filelog {filename parent {graphic {0}} } {
 #
 # This looks at the revision log of a file.  It's called from filebrowse.tcl, 
 # so we can't do operations such as merges.
@@ -1162,7 +1166,7 @@ proc cvs_filelog {filename parent} {
   global cvscfg
   global cwd
   
-  gen_log:log T "ENTER ($filename)"
+  gen_log:log T "ENTER ($filename $parent $graphic)"
   set pid [pid]
   set filetail [file tail $filename]
   
@@ -1176,10 +1180,15 @@ proc cvs_filelog {filename parent} {
     return
   }
 
-  set commandline "$cvs -d $cvscfg(cvsroot) log \"$filename\""
-
+  if {$graphic} {
   # Log canvas viewer
-  ::cvs_branchlog::new "CVS,rep" $filename
+    ::cvs_branchlog::new "CVS,rep" $filename
+  } else {
+    set commandline "$cvs -d $cvscfg(cvsroot) log \"$filename\""
+    set logcmd [viewer::new "CVS log $filename"]
+    $logcmd\::do "$commandline" 0 hilight_rcslog
+    $logcmd\::wait
+  }
   cd $cwd
   gen_log:log T "LEAVE"
 }
@@ -1286,6 +1295,7 @@ proc cvs_version_number {} {
   set number [$e\::output]
   regsub -all {\s*} $number {} number
   
+  $e\::destroy
   gen_log:log T "LEAVE ($number)"
   return $number
 }
@@ -1603,10 +1613,7 @@ proc cvs_ascii { args } {
   set cmd [exec::new "$cvs admin -kkv $filelist"]
   # gen_log:log D "Updating file list"
   # set cmd [exec::new "$cvs update $filelist"]
-  if {$cvscfg(auto_status)} {
-    $cmd\::wait
-    setup_dir
-  }
+  auto_setup_dir $cmd
 
   gen_log:log T "LEAVE"
 }
@@ -1630,10 +1637,7 @@ proc cvs_binary { args } {
   set cmd [exec::new "$cvs admin -kb $filelist"]
   # gen_log:log D "Updating file list"
   # set cmd [exec::new "$cvs update $filelist"]
-  if {$cvscfg(auto_status)} {
-    $cmd\::wait
-    setup_dir
-  }
+  auto_setup_dir $cmd
 
   gen_log:log T "LEAVE"
 }
@@ -1648,6 +1652,20 @@ proc cvs_revert {args} {
   gen_log:log T "ENTER ($args)"
   set filelist [join $args]
 
+  if {$filelist == ""} {
+    set mess "This will revert (discard) your changes to ** ALL ** files in this directory"
+  } else {
+    foreach file $filelist {
+      append revert_output "\n$file"
+    }
+    set mess "This will revert (discard) your changes to:$revert_output"
+  }
+  append mess "\n\nAre you sure?"
+
+  if {[cvsconfirm $mess .workdir] != "ok"} {
+    return 1
+  }
+
   gen_log:log D "Reverting $filelist"
   # update -C option appeared in 1.11
   set cvsglb(cvs_version) [cvs_version_number]
@@ -1661,10 +1679,7 @@ proc cvs_revert {args} {
     set cmd [exec::new "$cvs update -C $filelist"]
   }
   
-  if {$cvscfg(auto_status)} {
-    $cmd\::wait
-    setup_dir
-  }
+  auto_setup_dir $cmd
 
   gen_log:log T "LEAVE"
 }
@@ -1781,11 +1796,7 @@ proc cvs_lock {do files} {
     unlock { set commandline "cvs admin -u $files"}
   }
   set cmd [::exec::new "$commandline"]
-  
-  if {$cvscfg(auto_status)} {
-    $cmd\::wait
-    setup_dir
-  }
+  auto_setup_dir $cmd
 }
 
 # Sends directory "." to the directory-merge tool
@@ -1871,7 +1882,6 @@ proc cvs_branches {files} {
   foreach file $files {
     ::cvs_branchlog::new "CVS,loc" "$file"
   }
-
   gen_log:log T "LEAVE"
 }
 
@@ -1932,6 +1942,19 @@ namespace eval ::cvs_branchlog {
         }
       }
 
+      proc abortLog { } {
+        global cvscfg
+        variable cmd_log
+        variable lc
+
+        gen_log:log D "  $cmd_log\::abort"
+        catch {$cmd_log\::abort}
+        busy_done $lc
+        pack forget $lc.stop
+        pack $lc.close -in $lc.down.closefm -side right
+        $lc.close configure -state normal
+      }
+
       proc reloadLog { } {
         variable command
         variable cmd_log
@@ -1962,13 +1985,20 @@ namespace eval ::cvs_branchlog {
         catch { unset branchrevs }
         set cwd [pwd]
 
-        busy_start $lc
+        pack forget $lc.close
+        pack $lc.stop -in $lc.down.closefm -side right
+        $lc.stop configure -state normal
+
         set logstate {R}
 
         set cmd_log [::exec::new $command {} 0 [namespace current]::parse_cvslog]
         # wait for it to finish so our arrays are all populated
         $cmd_log\::wait
 
+        pack forget $lc.stop
+        pack $lc.close -in $lc.down.closefm -side right
+        $lc.close configure -state normal
+
         [namespace current]::cvs_sort_it_all_out
         gen_log:log T "LEAVE"
         return
@@ -1999,7 +2029,7 @@ namespace eval ::cvs_branchlog {
         variable rnum
         variable rootbranch
         variable revbranch
-        #gen_log:log T "ENTER ($exec $logline)"
+        gen_log:log T "ENTER ($exec $logline)"
 
         #gen_log:log D "$logline"
         if {$logline != {}} {
