@@ -20,8 +20,7 @@ proc cvs_execcmd {args} {
   # called for execmenu() entries.
   #
   gen_log:log T "ENTER ($args)"
-  gen_log:log C "$args"
-  eval "exec $args &"
+  exec::new $args
   gen_log:log T "LEAVE"
 }
 
@@ -41,19 +40,20 @@ proc cvs_catchcmd {args} {
 namespace eval ::exec {
   variable instance 0
 
-  proc new {command {viewer {}} {show_stderr {1}} {filter {}}} {
+  proc new {command {viewer {}} {show_stderr {1}} {filter {}} {errok {0}} } {
     variable instance
     set my_idx $instance
     incr instance
 
-    gen_log:log T "ENTER (\"$command\" \"$show_stderr\" \"$filter\")"
+    gen_log:log T "ENTER (\"$command\" \"$viewer\" \"$show_stderr\" \"$filter\" \"$errok\")"
 
     namespace eval $my_idx {
       set my_idx [uplevel {concat $my_idx}]
       variable command [uplevel {concat $command}]
+      variable show_stderr [uplevel {concat $show_stderr}]
       variable viewer [uplevel {concat $viewer}]
       variable filter [uplevel {concat $filter}]
-      variable show_stderr [uplevel {concat $show_stderr}]
+      variable errok [uplevel {concat $errok}]
 
       global cvscfg
       global errorCode
@@ -75,6 +75,7 @@ namespace eval ::exec {
         variable procerr
         variable ExecDone
         variable errmsg
+        variable errok
         variable data
         variable v_w
         variable my_idx
@@ -96,7 +97,7 @@ namespace eval ::exec {
             if {! [info exists command]} {set command ""}
             if {! [info exists status]} {set status ""}
             if {$errmsg == ""} {set errmsg "$command exited status $status"}
-            if {[string length $errmsg] < 512} {
+            if {[string length $errmsg] < 512 && ! $errok} {
                cvsfail $errmsg .
             }
             # If we don't pop up an error dialog, let's at least try to show
@@ -212,6 +213,12 @@ namespace eval ::exec {
         gen_log:log T "LEAVE"
       }
 
+      proc destroy {} {
+        gen_log:log T "ENTER"
+        catch {namespace delete [namespace current]}
+        gen_log:log T "LEAVE"
+      }
+
       proc wait {} {
         variable ExecDone
         gen_log:log T "ENTER"
@@ -329,6 +336,17 @@ namespace eval ::viewer {
         namespace inscope $v_e wait
       }
 
+      proc clean_exec {} {
+        variable v_e
+        catch {namespace inscope $v_e destroy}
+      }
+
+      proc destroy {} {
+        variable v_e
+        catch {namespace inscope $v_e destroy}
+        catch {namespace delete [namespace current]}
+      }
+
       # Call this proc to write arbitrary text to the viewer
       proc log { text {texttag {}} } {
         variable w
@@ -527,11 +545,12 @@ proc viewer_window {w title parent} {
   
   button $w.save -text "Save to File" -command "save_viewcontents $w"
   button $w.close -text "Close" -command "
-    namespace delete $parent
+    catch {$parent\::destroy}
     destroy $w
     exit_cleanup 0
   "
   button $w.stop -text "Stop" -bg red4 -fg white \
+      -activebackground red4 -activeforeground white \
       -state [expr {$cvscfg(allow_abort) ? {normal} : {disabled}}] \
       -command "$parent\::abort"
   pack $w.bottom -side bottom -fill x ;#-padx 25
