@@ -7,20 +7,23 @@ proc read_svn_dir {dirname} {
   global cmd
 
   gen_log:log T "ENTER ($dirname)"
-  # svn info gets the URL
+ # svn info gets the URL
   set cmd(info) [exec::new "svn info"]
   set info_lines [$cmd(info)\::output]
   foreach infoline [split $info_lines "\n"] {
-    if {[string match "URL:*" $infoline]} {
+    if {[string match "URL*" $infoline]} {
       #gen_log:log D "$infoline"
       set cvscfg(url) [lrange $infoline 1 end]
     }
   }
+  $cmd(info)\::destroy
+  catch {unset cmd(info)}
   if {$cvscfg(url) == ""} {
     cvsfail "Can't get the SVN URL"
     return
   }
 
+  set root ""
   foreach s {trunk branches tags} {
     if {[regexp "/$s/" $cvscfg(url)] || [regexp "/$s" $cvscfg(url)]} {
       set spl [split $cvscfg(url) "/"]
@@ -68,7 +71,17 @@ proc read_svn_dir {dirname} {
       gen_log:log D "tagname: $current_tagname"
     }
   }
-  gen_log:log T "LEAVE"
+  if {$root == ""} {
+    gen_log:log D "Nonconforming repository"
+    gen_log:log D "SVN URL: $cvscfg(url)"
+    set cvscfg(svnroot) $cvscfg(url)
+    gen_log:log D "svnroot: $cvscfg(svnroot)"
+    set cvsglb(relpath) ""
+    gen_log:log T "LEAVE (1)"
+    return 1
+  }
+  gen_log:log T "LEAVE (0)"
+  return 0
 }
 
 # Get stuff for main workdir browser
@@ -79,7 +92,10 @@ proc svn_workdir_status {} {
   gen_log:log T "ENTER"
   set cmd(svn_status) [exec::new "svn status -uvN"]
   set status_lines [split [$cmd(svn_status)\::output] "\n"]
-  catch {unset cmd(svn_status)}
+  if [info exists cmd(svn_status)] {
+    $cmd(svn_status)\::destroy
+    catch {unset cmd(svn_status)}
+  }
   # The first five columns in the output are each one character wide
   foreach logline $status_lines {
     if {[string match "Status*" $logline]} {continue}
@@ -157,11 +173,8 @@ proc svn_add {args} {
   if {$filelist == ""} {
     append filelist [glob -nocomplain $cvscfg(aster) .??*]
   }
-  set cmd(svnadd) [exec::new "svn add $filelist"]
-  if {$cvscfg(auto_status)} {
-    $cmd(svnadd)\::wait
-    setup_dir
-  }
+  set cmd [exec::new "svn add $filelist"]
+  auto_setup_dir $cmd
 
   gen_log:log T "LEAVE"
 }
@@ -174,11 +187,8 @@ proc svn_remove {args} {
   gen_log:log T "ENTER ($args)"
   set filelist [join $args]
 
-  set cmd(svndel) [exec::new "svn remove $filelist"]
-  if {$cvscfg(auto_status)} {
-    $cmd(svndel)\::wait
-    setup_dir
-  }
+  set cmd [exec::new "svn remove $filelist"]
+  auto_setup_dir $cmd
 
   gen_log:log T "LEAVE"
 }
@@ -245,11 +255,8 @@ proc svn_update {args} {
 
   set co_cmd [viewer::new "SVN Update"]
   $co_cmd\::do "$command" 0 status_colortags
-    
-  if {$cvscfg(auto_status)} {
-    $co_cmd\::wait
-    setup_dir
-  }
+  auto_setup_dir $co_cmd
+
   gen_log:log T "LEAVE"
 }
 
@@ -277,10 +284,7 @@ proc svn_opt_update {} {
   set upd_cmd [viewer::new "SVN Update/Switch"]
   $upd_cmd\::do "$command" 0 status_colortags
 
-  if {$cvscfg(auto_status)} {
-    $upd_cmd\::wait
-    setup_dir
-  }
+  auto_setup_dir $upd_cmd
 }
 
 # dialog for svn commit - called from workdir browser
@@ -455,7 +459,12 @@ proc svn_annotate_r {revision filepath} {
   global cvscfg
 
   gen_log:log T "ENTER ($revision $filepath)"
-  set revflag "-r$revision"
+  if {$revision != ""} {
+    # We were given a revision
+    set revflag "-r$revision"
+  } else {
+    set revflag ""
+  }
 
   annotate::new $revflag $filepath "svn_r"
   gen_log:log T "LEAVE"
@@ -483,8 +492,8 @@ proc svn_patch { pathA pathB revA dateA revB dateB outmode outfile } {
   } elseif {$dateA != {}} {
     set rev2 "\{\"$dateB\"\}"
   }
-set pathA [safe_url $pathA]
-set pathB [safe_url $pathB]
+  set pathA [safe_url $pathA]
+  set pathB [safe_url $pathB]
   if {$pathA != {} && $pathB != {}} {
     set command "svn diff $pathA $pathB"
   } elseif {$rev1 != {} && $rev2 != {}} {
@@ -507,6 +516,7 @@ set pathB [safe_url $pathB]
     }
     puts $fo $patch
     close $fo
+    $e\::destroy
     gen_log:log F "CLOSE $outfile"
   }
   gen_log:log T "LEAVE"
@@ -554,6 +564,8 @@ proc svn_jit_listdir { tf into } {
   set cmd(svnlist) [exec::new "$command"]
   if {[info exists cmd(svnlist)]} {
     set contents [split [$cmd(svnlist)\::output] "\n"]
+    $cmd(svnlist)\::destroy
+    catch {unset cmd(svnlist)}
   }
   set dirs {}
   set fils {}
@@ -604,6 +616,8 @@ proc svn_jit_dircmd { tf dir } {
   set cmd(svnlist) [exec::new "$command"]
   if {[info exists cmd(svnlist)]} {
     set contents [$cmd(svnlist)\::output]
+    $cmd(svnlist)\::destroy
+    catch {unset cmd(svnlist)}
   }
   set lbl "[file tail $dir]/"
   set exp "([llength $contents] items)"
@@ -654,6 +668,8 @@ proc parse_svnmodules {tf svnroot} {
   set cmd(svnlist) [exec::new "$command"]
   if {[info exists cmd(svnlist)]} {
     set contents [$cmd(svnlist)\::output]
+    $cmd(svnlist)\::destroy
+    catch {unset cmd(svnlist)}
   }
   set dirs {}
   set fils {}
@@ -708,7 +724,6 @@ proc svn_log {args} {
 
   set logcmd [viewer::new "SVN Log ($cvscfg(ldetail))"]
   $logcmd\::do "$command"
-  busy_done .workdir.main
   gen_log:log T "LEAVE"
 }
 
@@ -774,11 +789,7 @@ proc svn_revert {args} {
   }
   gen_log:log D "Reverting $filelist"
   set cmd [exec::new "svn revert $filelist"]
-
-  if {$cvscfg(auto_status)} {
-    $cmd\::wait
-    setup_dir
-  }
+  auto_setup_dir $cmd
 
   gen_log:log T "LEAVE"
 }
@@ -1092,9 +1103,14 @@ proc svn_branches {files} {
 }
 
 proc safe_url { url } {
-  regsub -all {%} $url {%25} url
-  regsub -all {&} $url {%26} url
-  regsub -all { } $url {%20} url
+  set url [string map {
+    "%" "%25"
+    "&" "%26"
+    " " "%20"
+  } $url]
+  #regsub -all {%} $url {%25} url
+  #regsub -all {&} $url {%26} url
+  #regsub -all { } $url {%20} url
   # These don't seem to be necessary
   #regsub -all {\+} $url {%2B} url
   #regsub -all {\-} $url {%2D} url
@@ -1140,6 +1156,30 @@ namespace eval ::svn_branchlog {
         set show_tags [set $ln\::opt(show_tags)]
       }
 
+      # Implementation of Perl-like "grep {/re/} in_list"
+      proc grep_filter { re in_list } {
+        set res ""
+        foreach x $in_list {
+          if {[regexp $re $x]} {
+            lappend res $x
+          }
+        }
+        return $res
+      }
+
+      proc abortLog { } {
+        global cvscfg
+        variable cmd_log
+        variable lc
+
+        gen_log:log D "  $cmd_log\::abort"
+        catch {$cmd_log\::abort}
+        busy_done $lc
+        pack forget $lc.stop
+        pack $lc.close -in $lc.down.closefm -side right
+        $lc.close configure -state normal
+      }
+
       proc reloadLog { } {
         global cvscfg
         global cvsglb
@@ -1178,6 +1218,10 @@ namespace eval ::svn_branchlog {
         catch { unset revpath }
         catch { unset revname }
 
+        pack forget $lc.close
+        pack $lc.stop -in $lc.down.closefm -side right
+        $lc.stop configure -state normal
+
         # Can't use file join or it will mess up the URL
         set safe_filename [safe_url $filename]
         set path "$cvscfg(url)/$safe_filename"
@@ -1185,87 +1229,107 @@ namespace eval ::svn_branchlog {
 
         # Find out where to put the working revision icon (if anywhere)
         set command "svn log -q --stop-on-copy \"$filename\""
-        set cmd [exec::new $command]
-        set log_output [$cmd\::output]
+        set cmd_log [exec::new $command]
+        set log_output [$cmd_log\::output]
+        $cmd_log\::destroy
         set loglines [split $log_output "\n"]
         set svnstat [lindex $loglines 1]
         set revnum_current [lindex $svnstat 0]
         gen_log:log D "revnum_current $revnum_current"
 
-        busy_start $lc
         if { $relpath == {} } {
           set path "$cvscfg(svnroot)/trunk/$safe_filename"
         } else {
           set path "$cvscfg(svnroot)/trunk/$relpath/$safe_filename"
         }
+        if {[read_svn_dir .] == 1} {
+          set path "$cvscfg(svnroot)/$safe_filename"
+          if {! [info exists cvscfg(svnconform_seen)]} {
+            set msg "Your repository does not seem to be arranged in trunk, branch, and root directories.  The Branch Browser can't detect branches and tags."
+            cvsok "$msg" $lc
+            set cvscfg(svnconform_seen) 1
+          }
+        }
         # The trunk
         set branchrevs(trunk) {}
         # if the file was added on a branch, this will error out.
         # Come to think of it, there's nothing especially privileged
-        #  about the trunk
+        # about the trunk except that one branch must not stop-on-copy
         set command "svn log $path"
-        gen_log:log C "$command"
-        set ret [catch {eval exec $command} log_output]
-        update idletasks
-        if {$ret == 0} {
-          set trunk_lines [split $log_output "\n"]
-          set rr [parse_svnlog $trunk_lines trunk]
-          # See if the current revision is on the trunk
-          set curr 0
-          set brevs $branchrevs(trunk)
-          set tip [lindex $brevs 0]
-          set revpath($tip) $path
-          set revkind($tip) "revision"
-          set brevs [lreplace $brevs 0 0]
-          if {$tip == $revnum_current} {
-            # If current is at end of trunk do this.
-            set branchrevs(trunk) [linsert $branchrevs(trunk) 0 {current}]
-            set curr 1
-          }
-          foreach r $brevs {
-            if {$r == $revnum_current} {
-              # We need to make a new artificial branch off of $r
-              lappend revbranches($r) {current}
-            }
-            gen_log:log D " $r $revdate($r) ($revcomment($r))"
-            set revkind($r) "revision"
-            set revpath($r) $path
-          }
-          #set branchrevs($rr) [lrange $branchrevs(trunk) 0 end-1]
-          set branchrevs($rr) $branchrevs(trunk)
-          set revkind($rr) "root"
-          set revname($rr) "trunk"
-          set revbtags($rr) "trunk"
-          set revpath($rr) $path
+        set cmd_log [exec::new $command {} 0 {} 1]
+        set log_output [$cmd_log\::output]
+        $cmd_log\::destroy
+        if {$log_output == ""} {
+          # Maybe the file isn't on the trunk anymore but it once was.
+          set j [string trimleft $revnum_current "r"]
+          set range "${j}:1"
+          set command "svn log -r $range $path"
+          set cmd_log [exec::new $command {} 0 {} 1]
+          set log_output [$cmd_log\::output]
+          $cmd_log\::destroy
         }
+        set trunk_lines [split $log_output "\n"]
+        set rr [parse_svnlog $trunk_lines trunk]
+        # See if the current revision is on the trunk
+        set curr 0
+        set brevs $branchrevs(trunk)
+        set tip [lindex $brevs 0]
+        set revpath($tip) $path
+        set revkind($tip) "revision"
+        set brevs [lreplace $brevs 0 0]
+        if {$tip == $revnum_current} {
+          # If current is at end of trunk do this.
+          set branchrevs(trunk) [linsert $branchrevs(trunk) 0 {current}]
+          set curr 1
+        }
+        foreach r $brevs {
+          if {$r == $revnum_current} {
+            # We need to make a new artificial branch off of $r
+            lappend revbranches($r) {current}
+          }
+          gen_log:log D " $r $revdate($r) ($revcomment($r))"
+          set revkind($r) "revision"
+          set revpath($r) $path
+        }
+        #set branchrevs($rr) [lrange $branchrevs(trunk) 0 end-1]
+        set branchrevs($rr) $branchrevs(trunk)
+        set revkind($rr) "root"
+        set revname($rr) "trunk"
+        set revbtags($rr) "trunk"
+        set revpath($rr) $path
 
         # Branches
         set command "svn list $cvscfg(svnroot)/branches"
-        gen_log:log C "$command"
-        set ret [catch {eval "exec $command"} branches]
-        if {$ret != 0} {
-          gen_log:log E "$branches"
-          set branches ""
+        set cmd_log [exec::new $command {} 0 {} 1]
+        set branches [$cmd_log\::output]
+        $cmd_log\::destroy
+
+        if {[info exists cvscfg(svn_branch_filter)] && \
+            [info exists cvscfg(svn_branch_max_count)]} {
+          # Only include branches that match regexp svn_branch_filter.
+          # And on top of that, keep only the top svn_branch_max_count.
+          set branches [grep_filter $cvscfg(svn_branch_filter) $branches]
+          set branches [lrange [lsort -decreasing $branches] \
+                      0 [expr {$cvscfg(svn_branch_max_count) - 1}]]
         }
+
         foreach branch $branches {
           gen_log:log D "$branch"
           # There can be files such as "README" here that aren't branches
           if {![string match {*/} $branch]} {continue}
           set branch [string trimright $branch "/"]
           # Can't use file join or it will mess up the URL
-          gen_log:log D "BRANCHES: RELPATH $relpath"
+          gen_log:log D "BRANCHES: RELPATH \"$relpath\""
           if { $relpath == {} } {
             set path "$cvscfg(svnroot)/branches/$branch/$safe_filename"
           } else {
             set path "$cvscfg(svnroot)/branches/$branch/$relpath/$safe_filename"
           }
           set command "svn log --stop-on-copy $path"
-          gen_log:log C "$command"
-          set ret [catch {eval exec $command} log_output]
-          update idletasks
-          if {$ret != 0} {
-            # This can happen a lot -let's not let it stop us
-            gen_log:log E "$log_output"
+          set cmd_log [exec::new $command {} 0 {} 1]
+          set log_output [$cmd_log\::output]
+          $cmd_log\::destroy
+          if {$log_output == ""} {
             continue
           }
           set loglines [split $log_output "\n"]
@@ -1299,11 +1363,11 @@ namespace eval ::svn_branchlog {
           set revpath($rb) $path
 
           set command "svn log -q $path"
-          gen_log:log C "$command"
-          set ret [catch {eval exec $command} log_output]
-          update idletasks
-          if {$ret != 0} {
-            cvsfail "$log_output"
+          set cmd_log [exec::new $command {} 0 {} 1]
+          set log_output [$cmd_log\::output]
+          $cmd_log\::destroy
+          if {$log_output == ""} {
+            cvsfail "$command returned no output"
             return
           }
           set loglines [split $log_output "\n"]
@@ -1317,17 +1381,13 @@ namespace eval ::svn_branchlog {
           }
           set bp [lindex $allrevs($branch) $idx]
           lappend revbranches($bp) $rb
-          update idletasks
         }
         # Tags
         if {$show_tags} {
           set command "svn list $cvscfg(svnroot)/tags"
-          gen_log:log C "$command"
-          set ret [catch {eval "exec $command"} tags]
-          if {$ret != 0} {
-              gen_log:log E "$tags"
-              set tags ""
-          }
+          set cmd_log [exec::new $command {} 0 {} 1]
+          set tags [$cmd_log\::output]
+          $cmd_log\::destroy
           set n_tags [llength $tags]
           if {$n_tags > $cvscfg(toomany_tags)} {
             # If confirm is on, give them a chance to say yes or no to tags
@@ -1345,9 +1405,9 @@ namespace eval ::svn_branchlog {
                 set tags ""
               }
             } else {
-            # Otherwise, just don't process tags
+              # Otherwise, just don't process tags
               set tags ""
-              gen_log:log E "Skipping tags: $n_tags > cvscfg(toomany_tags) ($cvscfg(toomany_tags)"
+              gen_log:log E "Skipping tags: $n_tags > cvscfg(toomany_tags) ($cvscfg(toomany_tags))"
             }
           }
           foreach tag $tags {
@@ -1356,19 +1416,18 @@ namespace eval ::svn_branchlog {
             if {![string match {*/} $tag]} {continue}
             set tag [string trimright $tag "/"]
             # Can't use file join or it will mess up the URL
-            gen_log:log D "TAGS: RELPATH $relpath"
+            gen_log:log D "TAGS: RELPATH \"$relpath\""
             if { $relpath == {} } {
               set path "$cvscfg(svnroot)/tags/$tag/$safe_filename"
             } else {
               set path "$cvscfg(svnroot)/tags/$tag/$relpath/$safe_filename"
             }
             set command "svn log --stop-on-copy $path"
-            gen_log:log C "$command"
-            set ret [catch {eval exec $command} log_output]
-            update idletasks
-            if {$ret != 0} {
-              # This can happen a lot -let's not let it stop us
-              gen_log:log E "$log_output"
+            set cmd_log [exec::new $command {} 0 {} 1]
+            set log_output [$cmd_log\::output]
+            $cmd_log\::destroy
+            #update idletasks
+            if {$log_output == ""} {
               continue
             }
             set loglines [split $log_output "\n"]
@@ -1383,11 +1442,11 @@ namespace eval ::svn_branchlog {
             set revpath($rb) $path
   
             set command "svn log -q $path"
-            gen_log:log C "$command"
-            set ret [catch {eval exec $command} log_output]
-            update idletasks
-            if {$ret != 0} {
-              cvsfail "$log_output"
+            set cmd_log [exec::new $command {} 0 {} 1]
+            set log_output [$cmd_log\::output]
+            $cmd_log\::destroy
+            if {$log_output == ""} {
+              cvsfail "$command returned no output"
               return
             }
             set loglines [split $log_output "\n"]
@@ -1398,6 +1457,10 @@ namespace eval ::svn_branchlog {
           }
         }
 
+        pack forget $lc.stop
+        pack $lc.close -in $lc.down.closefm -side right
+        $lc.close configure -state normal
+
         set branchrevs(current) {}
         [namespace current]::svn_sort_it_all_out
         gen_log:log T "LEAVE"
@@ -1411,6 +1474,8 @@ namespace eval ::svn_branchlog {
         variable revcomment
         variable branchrevs
 
+        gen_log:log T "ENTER (<...> $r)"
+        set revnum ""
         set i 0
         set l [llength $lines]
         while {$i < $l} {
@@ -1447,6 +1512,7 @@ namespace eval ::svn_branchlog {
           }
           incr i
         }
+        gen_log:log T "LEAVE \"$revnum\""
         return $revnum
       }
 
