@@ -30,7 +30,7 @@ proc workdir_setup {} {
   # Make a new toplevel and unmap . so that the working directory browser
   # the module browser are equal
   toplevel .workdir
-  wm title .workdir "TkCVS Working Directory"
+  wm title .workdir "TkCVS $cvscfg(version) Working Directory"
   wm iconname .workdir "TkCVS"
   if {$tcl_platform(platform) != "windows"} {
     wm iconbitmap .workdir @$cvscfg(bitmapdir)/tkcvs48.xbm
@@ -42,8 +42,6 @@ proc workdir_setup {} {
   if {[catch "image type Conflict"]} {
     workdir_images
   }
-  # Ignore the dimensions, because they depend on the content.  Use
-  # the remembered position though.
   if {[info exists cvscfg(workgeom)]} {
     wm geometry .workdir $cvscfg(workgeom)
   }
@@ -148,6 +146,8 @@ proc workdir_setup {} {
      -command { workdir_view_file [workdir_list_files] }
   button .workdir.bottom.buttons.funcs.bdelete_file -image Delete \
      -command { workdir_delete_file [workdir_list_files] }
+  button .workdir.bottom.buttons.funcs.bmkdir -image Dir_new \
+     -command { file_input_and_do "New Directory" workdir_newdir}
 
   button .workdir.bottom.buttons.dirfuncs.brefresh -image Refresh \
      -command { change_dir [pwd] }
@@ -198,9 +198,11 @@ proc workdir_setup {} {
 
   # These buttons work in any directory
   grid .workdir.bottom.buttons.funcs.bdelete_file -column 0 -row 0 \
-    -ipadx 2 -rowspan 4 -sticky ns
+    -ipadx 4
   grid .workdir.bottom.buttons.funcs.bedit_files -column 1 -row 0 \
      -ipadx 4
+  grid .workdir.bottom.buttons.funcs.bmkdir -column 0 -row 1 \
+     -ipadx 4
   grid .workdir.bottom.buttons.funcs.bview_files -column 1 -row 1 \
      -ipadx 4
 
@@ -254,6 +256,8 @@ proc workdir_setup {} {
      {"View the selected files"}
   set_tooltips .workdir.bottom.buttons.funcs.bdelete_file \
      {"Delete the selected files from the current directory"}
+  set_tooltips .workdir.bottom.buttons.funcs.bmkdir \
+     {"Make a new directory"}
 
   set_tooltips .workdir.bottom.buttons.dirfuncs.brefresh \
      {"Re-read the current directory"}
@@ -340,6 +344,8 @@ proc workdir_images {} {
     -format gif -file [file join $cvscfg(bitmapdir) annotate.gif]
   image create photo Delete \
     -format gif -file [file join $cvscfg(bitmapdir) delete.gif]
+  image create photo Dir_new \
+    -format gif -file [file join $cvscfg(bitmapdir) dir_new.gif]
   image create photo Refresh \
     -format gif -file [file join $cvscfg(bitmapdir) loop-glasses.gif]
   image create photo Branches \
@@ -442,7 +448,7 @@ proc workdir_menus {} {
      -command workdir_cleanup
   .workdir.menubar.file add separator
   .workdir.menubar.file add command -label "Shell window" -underline 0 \
-     -command {eval exec $cvscfg(shell) >& $cvscfg(null) &}
+     -command { exec::new $cvscfg(shell) }
   .workdir.menubar.file add separator
   .workdir.menubar.file add command -label Close -underline 1 \
      -command {.workdir.close invoke}
@@ -497,7 +503,7 @@ proc workdir_menus {} {
   .workdir.menubar.svn add command -label "Remove Files" -underline 0 \
      -command { subtract_dialog [workdir_list_files] }
   .workdir.menubar.svn add command -label "Browse the Log Diagram" \
-     -command { svn_branches [pwd] [workdir_list_files] }
+     -command { svn_branches [workdir_list_files] }
   .workdir.menubar.svn add separator
   .workdir.menubar.svn add command -label "Import CWD into Repository" \
      -underline 0 -command svn_import_run
@@ -679,7 +685,7 @@ proc workdir_edit_command {file} {
       }
     }
   }
-  return "\"$cvscfg(editor)\" $cvscfg(editorargs) \"$file\""
+  return "$cvscfg(editor) $cvscfg(editorargs) \"$file\""
 }
 
 proc workdir_newdir {file} {
@@ -717,9 +723,8 @@ proc workdir_edit_file {args} {
         # If the file doesn't exist it's tempting to touch the file and
         # trigger a reread, but is an empty file of this type valid?
         regsub -all {\$} $file {\$} file
-        set commandline "[workdir_edit_command $file] >& $cvscfg(null) &"
-        gen_log:log C "$commandline"
-        eval "exec $commandline"
+        set commandline [workdir_edit_command $file]
+        set editcmd [exec::new $commandline]
       } else {
         cvsfail "$file is not a plain file" .workdir
       }
@@ -866,6 +871,17 @@ proc change_dir {new_dir} {
   gen_log:log T "LEAVE"
 }
 
+proc auto_setup_dir {cmd} {
+  global cvscfg
+
+  if {$cvscfg(auto_status)} {
+    $cmd\::wait
+    setup_dir
+  } else {
+    after 0 "$cmd\::wait; $cmd\::destroy"
+  }
+}
+
 proc setup_dir { } {
   #
   # Call this when entering a directory.  It puts all of the file names
@@ -1098,11 +1114,12 @@ proc setup_dir { } {
     close $fileId
   } else {
     if {$insvn} {
-      set cmd(svnpropget) [exec::new "svn propget svn:ignore"]
-      set contents [split [$cmd(svnpropget)\::output] "\n"]
+      set cmd [exec::new "svn propget svn:ignore"]
+      set contents [split [$cmd\::output] "\n"]
       foreach line $contents {
         append cvscfg(ignore_file_filter) " $line"
       }
+      $cmd\::destroy
     }
   }
 
@@ -1335,14 +1352,8 @@ proc workdir_print_file {args} {
   if {[cvsconfirm $mess .workdir] == "ok"} {
     set final_result ""
     foreach file $filelist {
-      gen_log:log C "$cvscfg(print_cmd) \"$file\""
-      catch { eval exec $cvscfg(print_cmd) \"$file\" } file_result
-      if { $file_result != "" } {
-        set final_result "$final_result\n$file_result"
-      }
-    }
-    if { $final_result != "" } {
-      view_output::new "Print" $final_result
+      set commandline [concat $cvscfg(print_cmd) \"$file\"]
+      exec::new $commandline
     }
   }
   gen_log:log T "LEAVE"
@@ -1544,10 +1555,11 @@ proc save_options { } {
   # There are two kinds of options we can set
   set BOOLopts { allfiles auto_status confirm_prompt \
                  showstatcol showdatecol showeditcol auto_tag \
-                 status_filter recurse logging blame_linenums}
+                 status_filter recurse logging blame_linenums \
+                 svnconform_seen}
   set STRGopts { file_filter ignore_file_filter clean_these \
                  printer rdetail ldetail log_classes lastdir \
-                 workgeom modgeom loggeom tracgeom}
+                 workgeom modgeom loggeom tracgeom editor editorargs}
 
   # Plus the logcanvas options
   set LOGopts [concat [array names logcfg show_*] scale]
@@ -1558,6 +1570,14 @@ proc save_options { } {
   }
   set cvscfg(lastdir) [pwd]
 
+  if {[info exists cvscfg(editorargs)] } {
+    # editorargs is no longer necessary
+    if {$cvscfg(editorargs) != ""} {
+      set cvscfg(editor) [concat $cvscfg(editor) $cvscfg(editorargs)]
+    }
+    unset cvscfg(editorargs)
+  }
+
   # Save the list so we can keep track of what we've done
   set BOOLset $BOOLopts
   set STRGset $STRGopts
@@ -1617,6 +1637,10 @@ proc save_options { } {
             break
           }
         }
+        if {[string match "*set *cvscfg(editorargs)*" $line]} {
+          # editorargs is no longer necessary
+          continue
+        }
         foreach opt $LOGopts {
           if {! [info exists logcfg($opt)]} { continue }
           if {[string match "*set *logcfg($opt)*" $line]} {
